@@ -175,7 +175,7 @@ static int set_cap(envid_t veid, cap_t mask, int pid)
 	data.inheritable = mask;
 
 	if (capset(&header, &data) < 0)	{
-		logger(0, errno, "Unable to set capability");
+		logger(-1, errno, "Unable to set capability");
 		return VZ_SET_CAP;
 	}
 	return 0;
@@ -186,8 +186,8 @@ static inline cap_t make_cap_mask(cap_t on, cap_t off)
 	return (CAPDEFAULTMASK | on) & ~off;
 }
 
-/** Apply capability mask to VPS.
- * @param veid		VPS id.
+/** Apply capability mask to VE.
+ * @param veid		VE id.
  * @param cap		capability mask.
  * @return		0 on success.
  */
