@@ -42,6 +42,7 @@
 #include "meminfo.h"
 
 static int _page_size;
+static int check_name(char *name);
 
 static vps_config config[] = {
 /*	Op	*/
@@ -103,6 +104,7 @@ static vps_config config[] = {
 {"CPUUNITS",	NULL, PARAM_CPUUNITS},
 {"CPUUWEIGHT",	NULL, PARAM_CPUWEIGHT},
 {"CPULIMIT",	NULL, PARAM_CPULIMIT},
+{"CPUS",	NULL, PARAM_VCPUS},
 /* create param	*/
 {"ONBOOT",	NULL, PARAM_ONBOOT},
 {"CONFIGFILE",	NULL, PARAM_CONFIG},
@@ -116,6 +118,8 @@ static vps_config config[] = {
 {"QUOTAUGIDLIMIT",NULL, PARAM_QUOTAUGIDLIMIT},
 {"MEMINFO",	NULL, PARAM_MEMINFO},
 {"VETH",	NULL, PARAM_VETH_ADD},
+{"VEID",	NULL, PARAM_VEID},
+{"NAME",	NULL, PARAM_NAME},
 
 {NULL		,NULL, -1}
 };
@@ -123,6 +127,7 @@ static vps_config config[] = {
 static struct option set_opt[] = {
 {"save",	no_argument, NULL, PARAM_SAVE},
 {"applyconfig",	required_argument, NULL, PARAM_APPLYCONFIG},
+{"applyconfig_map",	required_argument, NULL, PARAM_APPLYCONFIG_MAP},
 {"config",	required_argument, NULL, PARAM_CONFIG},
 {"reset_ub",	no_argument, NULL, PARAM_RESET_UB},
 {"iptables",	required_argument, NULL, PARAM_IPTABLES},
@@ -176,6 +181,7 @@ static struct option set_opt[] = {
 {"cpuunits",	required_argument, NULL, PARAM_CPUUNITS},
 {"cpuweight",	required_argument, NULL, PARAM_CPUWEIGHT},
 {"cpulimit",	required_argument, NULL, PARAM_CPULIMIT},
+{"cpus",	required_argument, NULL, PARAM_VCPUS},
 /*	create param	*/
 {"onboot",	required_argument, NULL, PARAM_ONBOOT},
 {"setmode",	required_argument, NULL, PARAM_SETMODE},
@@ -189,6 +195,8 @@ static struct option set_opt[] = {
 /*	veth	*/
 {"veth_add",	required_argument, NULL, PARAM_VETH_ADD},
 {"veth_del",	required_argument, NULL, PARAM_VETH_DEL},
+/*	name	*/
+{"name",	required_argument, NULL, PARAM_NAME},
 
 {NULL, 0, NULL, 0}
 };
@@ -634,7 +642,7 @@ static int parse_cap(char *str, cap_param *cap)
                 return 0;
 	do {
 		if ((p = strrchr(token, ':')) == NULL) {
-			logger(0, 0, "Invalid syntaxes in %s:"
+			logger(-1, 0, "Invalid syntaxes in %s:"
 				" capname:on|off", token);
 			return ERR_INVAL;
 		}
@@ -643,7 +651,7 @@ static int parse_cap(char *str, cap_param *cap)
 		else if (!strcmp(p + 1, "on"))
 			mask = &cap->on;
 		else {
-                        logger(0, 0, "Invalid syntaxes in %s:"
+                        logger(-1, 0, "Invalid syntaxes in %s:"
 				" capname:on|off", token);
 			return ERR_INVAL;
 		}
@@ -652,7 +660,7 @@ static int parse_cap(char *str, cap_param *cap)
 			len < sizeof(cap_nm) ? len : sizeof(cap_nm));
 		cap_nm[len] = 0;
 		if (get_cap_mask(cap_nm, mask)) {
-			logger(0, 0, "Capability %s is unknown", cap_nm);
+			logger(-1, 0, "Capability %s is unknown", cap_nm);
 			return ERR_INVAL;
 		}
         } while ((token = strtok(NULL, " ")));
@@ -1050,7 +1058,7 @@ static int parse_devnodes_str(const char *str, dev_res *dev)
         snprintf(dev->name, len, "%s", str);
 	snprintf(buf, sizeof(buf), "/dev/%s", dev->name);
 	if (stat(buf, &st)) {
-		logger(0, errno, "Incorrect device name %s", buf);
+		logger(-1, errno, "Incorrect device name %s", buf);
 		return ERR_INVAL;
 	}
 	if (S_ISCHR(st.st_mode))
@@ -1058,7 +1066,7 @@ static int parse_devnodes_str(const char *str, dev_res *dev)
 	else if (S_ISBLK(st.st_mode))
 		dev->type = S_IFBLK;
 	else {
-		logger(0, 0, "The %s is not block or character device", buf);
+		logger(-1, 0, "The %s is not block or character device", buf);
 		return ERR_INVAL;
 	}
 	dev->dev = st.st_rdev;
@@ -1208,6 +1216,14 @@ static int store_cpu(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 			conf->name, *cpu->limit);
 		add_str_param(conf_h, buf);
 		break;
+	case PARAM_VCPUS:
+		if (cpu->vcpus == NULL)
+			break;
+		snprintf(buf, sizeof(buf), "%s=\"%lu\"",
+			conf->name, *cpu->vcpus);
+		add_str_param(conf_h, buf);
+		break;
+
 	}
 	return 0;
 }
@@ -1271,7 +1287,7 @@ static int parse_veth_str(const char *str, veth_dev *dev, int operation)
 	ch++;
 	tmp = ch;
 
-	/* Parsing veth name in VPS */
+	/* Parsing veth name in VE */
 	if ((ch = strchr(tmp, ',')) == NULL) 
 		return ERR_INVAL;
 	ch++;
@@ -1280,7 +1296,7 @@ static int parse_veth_str(const char *str, veth_dev *dev, int operation)
 		return ERR_INVAL;
 	snprintf(dev->dev_name_ve, len, "%s", tmp);
 
-	/* Parsing veth MAC address in VPS */
+	/* Parsing veth MAC address in VE */
 	len = strlen(ch);
 	if (len != 3*ETH_ALEN -1)
 		return ERR_INVAL;
@@ -1363,6 +1379,27 @@ static int store_veth(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 #undef STR2MAC
 }
 
+static int store_name(vps_param *old_p, vps_param *vps_p, vps_config *conf,
+        list_head_t *conf_h)
+{
+	char buf[STR_SIZE];
+
+	switch (conf->id) {
+	case PARAM_VEID:
+		if (vps_p->res.name.veid <= 0)
+			break;
+		snprintf(buf, sizeof(buf), "%s=\"%d\"", conf->name,
+			vps_p->res.name.veid);
+		add_str_param(conf_h, buf);
+		break;
+	case PARAM_NAME:
+		conf_store_str(conf_h, conf->name, vps_p->res.name.name);
+		add_str_param(conf_h, buf);
+		break;
+	}
+	return 0;
+}
+
 static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 {
 	int ret;
@@ -1406,6 +1443,12 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_USERPW:
 		ret = conf_parse_strlist(&vps_p->res.misc.userpw, val, 1);
 		break;
+	case PARAM_APPLYCONFIG_MAP:
+		if (!strcmp(val, "name"))
+			vps_p->opt.apply_cfg_map = APPCONF_MAP_NAME;
+		else
+			ret = ERR_INVAL;	
+		break;
 	case PARAM_APPLYCONFIG:
 		ret = conf_parse_str(&vps_p->opt.apply_cfg, val, 1);
 		break;
@@ -1570,6 +1613,16 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	        if (parse_cpulimit(&vps_p->res.cpu.limit, val))
         	        return ERR_INVAL;
 		break;
+	case PARAM_VCPUS:
+		if (vps_p->res.cpu.vcpus != NULL)
+			break;
+	        if (parse_ul(val, &uid))
+        	        return ERR_INVAL;
+		vps_p->res.cpu.vcpus = malloc(sizeof(unsigned long));
+		if (vps_p->res.cpu.vcpus == NULL)
+			return ERR_NOMEM;
+		*vps_p->res.cpu.vcpus = uid;
+		break;
 	case PARAM_MEMINFO:
 		ret = parse_meminfo(&vps_p->res.meminfo, val);
 		break;
@@ -1579,6 +1632,18 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_VETH_DEL:
 		ret = parse_veth(vps_p, val, 0);
 		break;
+	case PARAM_NAME:
+		if (vps_p->res.name.name != NULL)
+			break;
+		if (check_name(val))
+			return ERR_INVAL;
+		ret = conf_parse_str(&vps_p->res.name.name, val, 1);
+		break;
+	case PARAM_VEID:
+		if (parse_int(val, &int_id))
+			return ERR_INVAL;
+		vps_p->res.name.veid = int_id;
+		break;
 	default:
 		logger(10, 0, "Not handled parameter %d %s", id, val);
 		break;
@@ -1605,12 +1670,13 @@ static int store(vps_param *old_p, vps_param *vps_p, list_head_t *conf_h)
 		store_cpu(old_p, vps_p, conf, conf_h);
 		store_meminfo(old_p, vps_p, conf, conf_h);
 		store_veth(old_p, vps_p, conf, conf_h);
+		store_name(old_p, vps_p, conf, conf_h);
 	}
 	return 0;
 }
 
 /********************************************************/
-/*	VPS parse config stuff				*/
+/*	VE parse config stuff				*/
 /********************************************************/
 
 int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
@@ -1628,7 +1694,7 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 	const vps_config *conf;
 
 	if ((fp = fopen(path, "r")) == NULL) {
-		logger(0, errno, "Unable to open %s", path);
+		logger(-1, errno, "Unable to open %s", path);
                 return 1;
 	}
 	if (!stat(path, &st))
@@ -1654,22 +1720,22 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 		} else if (ret == ERR_INVAL_SKIP) {
 			continue;
 		} else if (ret == ERR_LONG_TRUNC) {
-			logger(0, 0, "Warning: too large value for %s=%s"
+			logger(-1, 0, "Warning: too large value for %s=%s"
 				" was truncated", ltoken, rtoken);
 		} else if (ret == ERR_DUP) {
-			logger(0, 0, "Warning: dup for %s=%s in line %d"
+			logger(-1, 0, "Warning: dup for %s=%s in line %d"
 				" is ignored", ltoken, rtoken, line);
 		} else if (ret == ERR_INVAL) {
-			logger(0, 0, "Invalid value for %s=%s, skipped",
+			logger(-1, 0, "Invalid value for %s=%s, skipped",
 				ltoken, rtoken);
 		} else if (ret == ERR_UNK) {
-			logger(0, 0, "Unknown parameter %s, skipped", ltoken);
+			logger(-1, 0, "Unknown parameter %s, skipped", ltoken);
 		} else if (ret == ERR_NOMEM) {
-			logger(0, 0, "Not enough memory");
+			logger(-1, 0, "Not enough memory");
 			err = VZ_RESOURCE_ERROR;
 			break;
 		} else {
-			logger(0, 0, "Unknown exit code %d on parse %s",
+			logger(-1, 0, "Unknown exit code %d on parse %s",
 				ret, ltoken);
 		}
 	}
@@ -1680,7 +1746,7 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 }
 
 /********************************************************/
-/*	VPS save config stuff				*/
+/*	VE save config stuff				*/
 /********************************************************/
 static int read_conf(char *fname, list_head_t *conf_h)
 {
@@ -1709,7 +1775,7 @@ static int write_conf(char *fname, list_head_t *head)
 	if (fname != NULL) {
 		snprintf(buf, sizeof(buf), "%s.tmp", fname);
 		if ((fd = open(buf, O_CREAT|O_WRONLY|O_TRUNC, 0644)) < 0) {
-			logger(0, errno, "Unable to create configuration"
+			logger(-1, errno, "Unable to create configuration"
 				" file %s", buf);
 	                return 1;
         	}
@@ -1720,7 +1786,7 @@ static int write_conf(char *fname, list_head_t *head)
 		len = strlen(conf->val);
 		ret = write(fd, conf->val, len);
 		if (ret < 0) {
-			logger(0, errno, "Unable to write %d bytes to %s",
+			logger(-1, errno, "Unable to write %d bytes to %s",
 					len, buf);
 			unlink(buf);
 			close(fd);
@@ -1732,7 +1798,7 @@ static int write_conf(char *fname, list_head_t *head)
 	if (fname != NULL) {
 		close(fd);
 		if (rename(buf, fname)) {
-			logger(0, errno, "Unable to move %s -> %s",
+			logger(-1, errno, "Unable to move %s -> %s",
 				buf, fname);
 			return 1;
 		}
@@ -1867,6 +1933,87 @@ vps_param *init_vps_param()
 	return param;
 }
 
+int get_veid_by_name(char *name)
+{
+	char buf[64];
+	char content[256];
+	char *p;
+	int veid, ret;
+
+	if (name == NULL)
+		return -1;
+	snprintf(buf, sizeof(buf), VENAME_DIR "/%s", name);
+	if (stat_file(buf) != 1)
+		return -1;
+	ret = readlink(buf, content, sizeof(content) - 1);
+	if (ret < 0)
+		return -1;
+	content[ret] = 0;
+	if ((p = strrchr(content, '/')) == NULL)
+		p = content;
+	else
+		p++;
+	if (sscanf(p, "%d.conf", &veid) == 1)
+		return veid;
+	return -1;
+}
+
+static int check_name(char *name)
+{
+        char *p;
+
+        for (p = name; *p != '\0'; p++) {
+                if (!isdigit(*p) && !isalpha(*p) && *p != '-' && *p != '_')
+                        return -1;
+	}
+        return 0;
+}
+
+int set_name(int veid, char *new_name, char *old_name)
+{
+	int veid_old = -1;
+	char buf[STR_SIZE];
+	char conf[STR_SIZE];
+
+	if (new_name == NULL)
+		return 0;
+	if (check_name(new_name)) {
+		logger(-1, 0, "Error: invalid name %s", new_name);
+		return VZ_SET_NAME_ERROR;
+	}
+	veid_old = get_veid_by_name(new_name);
+	if (veid_old >= 0 && veid_old != veid) {
+		logger(-1, 0, "Conflict: name %s already used by VE %d",
+			new_name, veid_old);
+		return VZ_SET_NAME_ERROR;
+	}
+	if (old_name != NULL &&
+		!strcmp(old_name, new_name) &&
+		veid == veid_old)
+	{
+		return 0;	
+	}
+	if (new_name[0] != 0) {
+		snprintf(buf, sizeof(buf), VENAME_DIR "/%s", new_name);
+		get_vps_conf_path(veid, conf, sizeof(conf));
+		unlink(buf);
+		if (symlink(conf, buf)) {
+			logger(-1, errno, "Unable to create link %s", buf);
+			return VZ_SET_NAME_ERROR;
+		}
+	}
+	veid_old = get_veid_by_name(old_name);
+	/* Remove old alias */
+	if (veid_old == veid &&
+	    old_name != NULL && strcmp(old_name, new_name))
+	{
+		snprintf(buf, sizeof(buf), VENAME_DIR "/%s", old_name);
+		unlink(buf);
+	}
+	logger(0, 0, "Name %s assigned", new_name);
+	return 0;
+}
+
 #define FREE_P(x)	if ((x) != NULL) { free(x); x = NULL; }
 
 static void free_opt(vps_opt *opt)
@@ -1929,6 +2076,7 @@ static void free_cpu(cpu_param *cpu)
 	FREE_P(cpu->units)
 	FREE_P(cpu->weight)
 	FREE_P(cpu->limit)
+	FREE_P(cpu->vcpus)
 }
 
 static void free_dq(dq_param *dq)
@@ -1944,6 +2092,11 @@ static void free_cpt(cpt_param *cpt)
 	FREE_P(cpt->dumpdir)
 	FREE_P(cpt->dumpfile)
 }
+
+static void free_name(name_param *name)
+{
+	FREE_P(name->name)
+}
 #undef FREE_P
 
 static void free_vps_res(vps_res *res)
@@ -1958,6 +2111,7 @@ static void free_vps_res(vps_res *res)
 	free_dq(&res->dq);
 	free_cpt(&res->cpt);
 	free_veth_param(&res->veth);
+	free_name(&res->name);
 }
 
 void free_vps_param(vps_param *param)
@@ -2006,6 +2160,7 @@ static void merge_opt(vps_opt *dst, vps_opt *src)
 	MERGE_INT(start_force);
 	MERGE_INT(onboot);
 	MERGE_INT(setmode);
+	MERGE_INT(apply_cfg_map);
 
 	MERGE_STR(config)
 	MERGE_STR(origin_sample)
@@ -2036,6 +2191,7 @@ static void merge_cpu(cpu_param *dst, cpu_param *src)
 	MERGE_P(units)
 	MERGE_P(weight)
 	MERGE_P(limit)
+	MERGE_P(vcpus)
 }
 
 #define	MERGE_LIST(x)							\
@@ -2058,6 +2214,7 @@ static void merge_misc(misc_param *dst, misc_param *src)
 	MERGE_LIST(searchdomain)
 	MERGE_LIST(userpw)
 	MERGE_STR(hostname)
+	MERGE_INT(wait);
 }
 
 static void merge_dq(dq_param *dst, dq_param *src)
@@ -2117,6 +2274,12 @@ static void merge_veth(veth_param *dst, veth_param *src)
 	}
 }
 
+static void merge_name(name_param *dst, name_param *src)
+{
+	MERGE_STR(name);
+	MERGE_INT(veid);
+}
+
 static int merge_res(vps_res *dst, vps_res *src)
 {
 	merge_fs(&dst->fs, &src->fs);
@@ -2132,6 +2295,7 @@ static int merge_res(vps_res *dst, vps_res *src)
 	merge_cpt(&dst->cpt, &src->cpt);
 	merge_meminfo(&dst->meminfo, &src->meminfo);
 	merge_veth(&dst->veth, &src->veth);
+	merge_name(&dst->name, &src->name);
 	return 0;
 }
 
