@@ -33,20 +33,39 @@
 #define __NR_fairsched_chwt    1502
 #elif __x86_64__
 #define __NR_fairsched_chwt    506
+#elif __powerpc__
+#define __NR_fairsched_chwt    402
 #else
 #define __NR_fairsched_chwt    502
 #endif
 #endif
+
 #ifndef __NR_fairsched_rate
 #ifdef __ia64__
 #define __NR_fairsched_rate    1504
 #elif __x86_64__
 #define __NR_fairsched_rate    508
+#elif __powerpc__
+#define __NR_fairsched_rate    404
 #else
 #define __NR_fairsched_rate    504
 #endif
 #endif
 
+#ifndef __NR_fairsched_vcpus
+#ifdef __i386__
+#define __NR_fairsched_vcpus    505
+#elif __x86_64__
+#define __NR_fairsched_vcpus    499
+#elif __ia64__
+#define __NR_fairsched_vcpus    1499
+#elif __powerpc__
+#define __NR_fairsched_vcpus    405
+#else
+#error "no syscall for this arch"
+#endif
+#endif
+
 static inline int fairsched_chwt(unsigned int id, unsigned wght)
 {
 	int ret;
@@ -67,13 +86,23 @@ static inline int fairsched_rate(unsigned int id, int op, unsigned rate)
 	return ret;
 }
 
+static inline int fairsched_vcpus(unsigned int id, unsigned vcpus)
+{
+	int ret;
+
+	ret = syscall(__NR_fairsched_vcpus, id, vcpus);
+	if (ret && errno == ENOSYS)
+		ret = 0;
+	return ret;
+}
+
 static int set_cpulimit(envid_t veid, unsigned int cpulimit)
 {
 	unsigned cpulim1024 = (float)cpulimit * 1024 / 100;
 
 	logger(0, 0, "Setting CPU limit: %d", cpulimit);
 	if (fairsched_rate(veid, cpulim1024 ? 0 : 1, cpulim1024) < 0) {
-		logger(0, errno, "fairsched_rate");
+		logger(-1, errno, "fairsched_rate");
 		return VZ_SETFSHD_ERROR;
 	}
 	return 0;
@@ -83,7 +112,7 @@ static int set_cpuweight(envid_t veid, unsigned int cpuweight)
 {
 
 	if (fairsched_chwt(veid, cpuweight)) {
-		logger(0, errno, "fairsched_chwt");
+		logger(-1, errno, "fairsched_chwt");
 		return VZ_SETFSHD_ERROR;
 	}
 	return 0;
@@ -94,7 +123,7 @@ static int set_cpuunits(envid_t veid, unsigned int cpuunits)
 	int cpuweight, ret;
 
 	if (cpuunits < MINCPUUNITS || cpuunits > MAXCPUUNITS) {
-		logger(0, 0, "Invalid value for cpuunits: %d"
+		logger(-1, 0, "Invalid value for cpuunits: %d"
 			" allowed range is %d-%d",
 			cpuunits, MINCPUUNITS, MAXCPUUNITS);
 		return VZ_SETFSHD_ERROR;
@@ -105,6 +134,24 @@ static int set_cpuunits(envid_t veid, unsigned int cpuunits)
 	return ret;
 }
 
+/** Change number of CPUs available in the running VE.
+ *
+ * @param veid		VE id
+ * @param vcpu		number of cpu
+ */
+int env_set_vcpus(envid_t veid, unsigned int vcpus)
+{
+	int ret;
+
+	logger(0, 0, "Setting CPUs: %d", vcpus);
+	ret = fairsched_vcpus(veid, vcpus);
+	if (ret) 
+		logger(-1, errno, "Unable to set cpus");
+
+	return ret;
+}
+
+
 /**  Apply cpu parameters on Host system.
  *
  * @param cpu		cpu parameters.
@@ -117,10 +164,10 @@ int hn_set_cpu(cpu_param *cpu)
 	return set_cpuunits(2147483647, *cpu->units);
 }
 
-/**  Apply cpu parameters on running VPS.
+/**  Apply cpu parameters on running VE.
  *
- * @param h		VPS handler.
- * @param veid		VPS id.
+ * @param h		VE handler.
+ * @param veid		VE id.
  * @param cpu		cpu parameters.
  * @return		0 on success.
  */
@@ -130,13 +177,14 @@ int vps_set_cpu(vps_handler *h, envid_t veid, cpu_param *cpu)
 
 	if (cpu->limit == NULL &&
 		cpu->units == NULL &&
-		cpu->weight == NULL)
+		cpu->weight == NULL &&
+		cpu->vcpus == NULL)
 	{
 		return 0;
 	}
 	if (!vps_is_run(h, veid)) {
-		logger(0, 0, "Unable to apply CPU parameters: VPS is not"
-			" running");
+		logger(-1, 0, "Unable to apply CPU parameters: "
+			"VE is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	if (cpu->limit != NULL) {
@@ -146,6 +194,11 @@ int vps_set_cpu(vps_handler *h, envid_t veid, cpu_param *cpu)
 		ret = set_cpuunits(veid, *cpu->units);
 	} else if (cpu->weight != NULL)
 		ret = set_cpuweight(veid, *cpu->weight);
+	if (cpu->vcpus != NULL) {
+		ret = env_set_vcpus(veid, *cpu->vcpus);
+	}
 
 	return ret;
 }
+
+
