@@ -49,6 +49,8 @@
 #define __NR_setluid	1506
 #elif __x86_64__
 #define __NR_setluid	501
+#elif __powerpc__
+#define __NR_setluid    411
 #else
 #define __NR_setluid	511
 #endif
@@ -95,7 +97,7 @@ int vz_env_create_data_ioctl(vps_handler *h,
 		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE_DATA, data);
 	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
 #ifdef  __x86_64__
-	/* Set personality PER_LINUX32 for i386 based VPSes */
+	/* Set personality PER_LINUX32 for i386 based VEs */
 	if (errcode >= 0)
 		set_personality32();
 #endif
@@ -117,7 +119,7 @@ int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
 		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE, &env_create);
 	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
 #ifdef  __x86_64__
-	/* Set personality PER_LINUX32 for i386 based VPSes */
+	/* Set personality PER_LINUX32 for i386 based VEs */
 	if (errcode >= 0 && (flags & VE_ENTER))
 		set_personality32();
 #endif
@@ -145,9 +147,9 @@ static int reset_std()
 	return stdfd;
 }
 
-/** Allocate and inittialize VPS handler.
+/** Allocate and initialize VE handler.
  *
- * @param veid		VPS id.
+ * @param veid		VE id.
  * @return		handler or NULL on error.
  */
 vps_handler *vz_open(envid_t veid)
@@ -157,8 +159,8 @@ vps_handler *vz_open(envid_t veid)
 
 	stdfd = reset_std();
         if ((vzfd = open(VZCTLDEV, O_RDWR)) < 0) {
-                logger(0, errno, "Unable to open %s", VZCTLDEV);
-                logger(0, 0, "Please check that vzdev kernel module is loaded"
+                logger(-1, errno, "Unable to open %s", VZCTLDEV);
+                logger(-1, 0, "Please check that vzdev kernel module is loaded"
                         " and you have sufficient permissions"
                         " to access the file.");
 		goto err;
@@ -171,7 +173,7 @@ vps_handler *vz_open(envid_t veid)
         if (vz_env_create_ioctl(h, 0, 0) < 0 &&
 		(errno == ENOSYS || errno == EPERM))
         {
-                logger(0, 0, "Your kernel lacks support for virtual"
+                logger(-1, 0, "Your kernel lacks support for virtual"
                                 " environments or modules not loaded");
 		goto err;
 	}
@@ -187,9 +189,9 @@ err:
 	return NULL;
 }
 
-/** Close VPS handler.
+/** Close VE handler.
  *
- * @param h		VPS handler.
+ * @param h		VE handler.
  */
 void vz_close(vps_handler *h)
 {
@@ -201,12 +203,12 @@ void vz_close(vps_handler *h)
 	free(h);
 }
 
-/** Get VPS status.
+/** Get VE status.
  *
- * @param h		VPS handler.
- * @param veid		VPS id.
- * @return		1 - VPS is running
- *			0 - VPS is stopped.
+ * @param h		VE handler.
+ * @param veid		VE id.
+ * @return		1 - VE is running
+ *			0 - VE is stopped.
  */
 int vps_is_run(vps_handler *h, envid_t veid)
 {
@@ -220,13 +222,13 @@ int vps_is_run(vps_handler *h, envid_t veid)
 	if (ret < 0 && (errno == ESRCH || errno == ENOTTY))
 		return 0;
 	else if (ret < 0)
-		logger(0, errno, "error on vz_env_create_ioctl(VE_TEST)");
+		logger(-1, errno, "error on vz_env_create_ioctl(VE_TEST)");
         return 1;
 }
 
 /** Change root to specified directory
  *
- * @param		VPS root
+ * @param		VE root
  * @return		0 on success
  */
 int vz_chroot(char *root)
@@ -236,16 +238,16 @@ int vz_chroot(char *root)
 	struct sigaction act;
 
 	if (root == NULL) {
-		logger(0, 0, "vz_chroot: VPS root is not specified");	
+		logger(-1, 0, "vz_chroot: VE root is not specified");	
 		return VZ_VE_ROOT_NOTSET;
 	}
 	if (chdir(root)) {
-		logger(0, errno, "unable to change dir to %s",
+		logger(-1, errno, "unable to change dir to %s",
 			root);
 		return VZ_RESOURCE_ERROR;
 	}
 	if (chroot(root)) {
-		logger(0, errno, "chroot %s failed", root);
+		logger(-1, errno, "chroot %s failed", root);
 		return VZ_RESOURCE_ERROR;
 	}
 	setsid();
@@ -261,15 +263,20 @@ int vz_chroot(char *root)
 
 int vz_setluid(envid_t veid)
 {
-        if (setluid(veid) == -1) 
+        if (setluid(veid) == -1) {
+		if (errno == ENOSYS)
+			logger(-1, 0, "Error: kernel does not support"
+				" user resources. Please, rebuild with"
+				" CONFIG_USER_RESOURCE=y");
                 return VZ_SETLUID_ERROR;
+	}
         return 0;
 }
 
 /*
- * Checks if sysfs needs to be enabled for this VPS.
+ * Checks if sysfs needs to be enabled for this VE.
  * Now we do that only for distributions from sysfs_dists.
- * FIXME: priovide a generic way to enable/disable sysfs per VPS
+ * FIXME: provide a generic way to enable/disable sysfs per VE.
 */
 static int sysfs_required(vps_res *res)
 {
@@ -360,7 +367,7 @@ try:
 				ret = VZ_BAD_KERNEL;
 				break;
 			default:
-				logger(0, errno, "env_create error");
+				logger(-1, errno, "env_create error");
 				ret = VZ_ENVCREATE_ERROR;
 				break;
 		}
@@ -374,11 +381,17 @@ try:
 	mk_reboot_script();
 	if (res->dq.ugidlimit != NULL)
 		mk_quota_link();
+	if (res->misc.wait == YES) {
+		if (add_reach_runlevel_mark()) {
+			ret = VZ_WAIT_FAILED;
+			goto env_err;
+		}
+	}
 	/* Close status descriptor to report that
 	 * environment is created.
 	*/
 	close(STDIN_FILENO);
-	/* Now we wait until VPS setup will be done
+	/* Now we wait until VE setup will be done
 	   If no error start init otherwise exit.
 	*/
 	if (read(wait_p, &ret, sizeof(ret)) != 0)
@@ -407,7 +420,7 @@ static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		return ret;
 	/* Create another process to proper resource accounting */
 	if ((pid = fork()) < 0) {
-		logger(0, errno, "Unable to fork");
+		logger(-1, errno, "Unable to fork");
 		return VZ_RESOURCE_ERROR;
 	} else if (pid == 0) {
 		if ((ret = vps_set_cap(veid, &res->cap)))
@@ -435,7 +448,7 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (pipe(status_p) < 0) {
-		logger(0, errno, "Can not create pipe");
+		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
 	}
 	sigaction(SIGCHLD, NULL, &actold);
@@ -445,7 +458,7 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
         sigaction(SIGCHLD, &act, NULL);
 
 	if ((pid = fork()) < 0) {
-		logger(0, errno, "Can not fork");
+		logger(-1, errno, "Can not fork");
 		ret =  VZ_RESOURCE_ERROR;
 		goto err;
 	} else if (pid == 0) {
@@ -473,19 +486,23 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		ret = errcode;
 		switch(ret) {
 		case VZ_NO_ACCES:
-			logger(0,0, "Permission denied");
+			logger(-1, 0, "Permission denied");
 			break;
 		case VZ_BAD_KERNEL:
-			logger(0, 0, "Invalid kernel, or some kernel"
+			logger(-1, 0, "Invalid kernel, or some kernel"
 				" modules are not loaded");
 			break;
 		case VZ_SET_CAP:
-			logger(0, 0, "Unable to set capability");
+			logger(-1, 0, "Unable to set capability");
 			break;
 		case VZ_RESOURCE_ERROR:
-			logger(0, 0, "Not enough resources"
+			logger(-1, 0, "Not enough resources"
 				" to start environment");
 			break;
+		case VZ_WAIT_FAILED:
+			logger(0, 0, "Unable to set"
+				" wait functionality");
+			break;
 		}
         }
 err:
@@ -504,7 +521,7 @@ static void fix_numiptent(ub_param *ub)
 		return;
 	min_ipt = min_ul(ub->numiptent[0], ub->numiptent[1]);
 	if (min_ipt < MIN_NUMIPTENT) {
-		logger(0, 0, "Warning: NUMIPTENT %d:%d is less"
+		logger(-1, 0, "Warning: NUMIPTENT %d:%d is less"
 			" than minimally allowable value, set to %d:%d",
 			ub->numiptent[0], ub->numiptent[1],
 			MIN_NUMIPTENT, MIN_NUMIPTENT);
@@ -527,7 +544,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 {
 	int wait_p[2];
 	int err_p[2];
-	int ret;
+	int ret, err;
 	char buf[64];
 	char *dist_name;
 	struct sigaction act;
@@ -538,7 +555,7 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (vps_is_run(h, veid)) {
-		logger(0, 0, "VPS is already running");
+		logger(-1, 0, "VE is already running");
 		return VZ_VE_RUNNING;
 	}
 	if ((ret = check_ub(&res->ub)))
@@ -549,9 +566,9 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		free(dist_name);
 	if (ret)
 		return ret;
-	logger(0, 0, "Starting VPS ...");
+	logger(0, 0, "Starting VE ...");
 	if (vps_is_mounted(res->fs.root)) {
-		/* if VPS mounted umount first, to cleanup mount state */
+		/* if VE is mounted -- umount first, to cleanup mount state */
 		vps_umount(h, veid, res->fs.root, skip);
 	}
 	if (!vps_is_mounted(res->fs.root)) {
@@ -562,13 +579,13 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		quouta_inc(&res->dq, -100);
 	}
 	if (pipe(wait_p) < 0) {
-		logger(0, errno, "Can not create pipe");
+		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
 	}
 	if (pipe(err_p) < 0) {
 		close(wait_p[0]);
 		close(wait_p[1]);
-		logger(0, errno, "Can not create pipe");
+		logger(-1, errno, "Can not create pipe");
 		return VZ_RESOURCE_ERROR;
 	}
 	sigemptyset(&act.sa_mask);
@@ -598,22 +615,37 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 	}
 	/* Close fd to start /sbin/init */
 	if (close(wait_p[1]))
-		logger(0, errno, "Unable to close fd to start init");
+		logger(-1, errno, "Unable to close fd to start init");
 err:
 	free_dist_actions(&actions);
 	if (ret) {
-		int err;
 		/* Kill enviroment */
-		logger(0, 0, "VPS start failed");
+		logger(-1, 0, "VE start failed");
 		write(wait_p[1], &err, sizeof(err));
 	} else {
 		if (!read(err_p[0], &ret, sizeof(ret))) {
-                        logger(0, 0, "VPS start in progress...");
+			if (res->misc.wait == YES) {
+				logger(0, 0, "VE start in progress"
+					" waiting ...");
+				err = vps_execFn(h, veid, res->fs.root,
+					wait_on_fifo, NULL, 0);
+				if (err) {
+					logger(0, 0, "VE wait failed %s",
+						err == VZ_EXEC_TIMEOUT ? \
+						"timeout expired" : "");
+					ret = VZ_WAIT_FAILED;
+				} else {
+					logger(0, 0, "VE started"
+						" successfully");
+				}
+			} else {
+	                        logger(0, 0, "VE start in progress...");
+			}
 		} else {
 			if (ret == VZ_FS_BAD_TMPL)
-				logger(0, 0, "Unable to start init, probably"
+				logger(-1, 0, "Unable to start init, probably"
 					" incorrect template");
-                        logger(0, 0, "VPS start failed");
+                        logger(-1, 0, "VE start failed");
                 }
 	}
 	if (ret) {
@@ -632,13 +664,13 @@ err:
 	return ret;
 }
 
-/** Sart and configure VPS.
+/** Start and configure VE.
  *
- * @param h		VPS handler.
- * @param veid		VPS id.
- * @param res		VPS resourses.
+ * @param h		VE handler.
+ * @param veid		VE id.
+ * @param res		VE resourses.
  * @param d_actions	distribution specific sctions.
- * @param skip		flags to skip VPS setup (SKIP_SETUP|SKIP_ACTION_SCRIPT)
+ * @param skip		flags to skip VE setup (SKIP_SETUP|SKIP_ACTION_SCRIPT)
  * @param action	modules list, used to call setup() callback
  * @return		0 on success.
  */
@@ -661,7 +693,7 @@ static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
 	if ((ret = vz_env_create_ioctl(h, veid, VE_ENTER)) < 0) {
 		if (errno == ESRCH)
 			return 0;
-		logger(0, errno, "VPS enter failed");
+		logger(-1, errno, "VE enter failed");
 		return ret;
 	}
 	close(h->vzfd);
@@ -700,11 +732,11 @@ static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
 	act.sa_flags = SA_NOCLDSTOP;
         sigaction(SIGCHLD, &act, NULL);
 
-	logger(0, 0, "Stopping VPS ...");
+	logger(0, 0, "Stopping VE ...");
 	if (stop_mode == M_KILL)
 		goto kill_vps;
 	if ((pid = fork()) < 0) {
-		logger(0, errno, "Can not fork");
+		logger(-1, errno, "Can not fork");
 		ret = VZ_RESOURCE_ERROR;
 		goto out;
 	} else if (pid == 0) {
@@ -722,7 +754,7 @@ static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
 kill_vps:
         if ((pid = fork()) < 0) {
 		ret = VZ_RESOURCE_ERROR;
-		logger(0, errno, "Can not fork");
+		logger(-1, errno, "Can not fork");
 		goto err;
 	
 	} else if (pid == 0) {
@@ -739,19 +771,19 @@ kill_vps:
 	}
 out:
 	if (ret)
-		logger(0, 0, "Unable to stop VPS, operation timed out");
+		logger(-1, 0, "Unable to stop VE: operation timed out");
 	else
-		logger(0, 0, "VPS was stopped");
+		logger(0, 0, "VE was stopped");
 err:
 	sigaction(SIGCHLD, &actold, NULL);
 	return ret;
 }
 
-/** Stop VPS.
+/** Stop VE.
  *
- * @param h		VPS handler.
- * @param veid		VPS id.
- * @param res		VPS resourses.
+ * @param h		VE handler.
+ * @param veid		VE id.
+ * @param res		VE resourses.
  * @param stop_mode	stop mode one of (M_REBOOT M_HALT M_KILL).
  * @param skip		flags to skip run action script (SKIP_ACTION_SCRIPT)
  * @param action	modules list, used to call cleanup() callback.
@@ -767,7 +799,7 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	if (check_var(res->fs.root, "VE_ROOT is not set"))
 		return VZ_VE_ROOT_NOTSET;
 	if (!vps_is_run(h, veid)) {
-		logger(0, 0, "Unable to stop: VPS is not running");
+		logger(-1, 0, "Unable to stop: VE is not running");
 		return 0;
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
@@ -781,16 +813,18 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 			}
 		}
 	}
-	/* get VPS ip adresses for cleanup */
+	/* get VE IP addresses for cleanup */
 	get_vps_ip(h, veid, &param->del_res.net.ip);
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
-		return ret;
+		goto end;
 	mod_cleanup(h, veid, action, param);
 	vps_cleanup_res(h, veid, param, STATE_STOPPING);
 	ret = vps_umount(h, veid, res->fs.root, skip);
-	/* Clear VPS network configuration*/
+	/* Clear VE network configuration*/
 	ret = run_pre_script(veid, VPS_STOP);
 
+end:
+	free_str_param(&param->del_res.net.ip);
 	return ret;
 }
 
@@ -798,7 +832,7 @@ int vps_restart(vps_handler *h, envid_t veid, vps_param *param)
 {
 	int ret;
 
-	logger(0, 0, "Restarting VPS");
+	logger(0, 0, "Restarting VE");
 	if (vps_is_run(h, veid) &&
 		(ret = vps_stop(h, veid, param, M_REBOOT, SKIP_NONE, NULL)))
 	{
