@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <stdlib.h>
@@ -32,22 +39,27 @@ int vz_fs_is_mounted(char *root)
 	FILE *fp;
 	char buf[512];
 	char mnt[512];
-	int ret = 0; 
+	char *path;
+	int ret = 0;
 
 	if ((fp = fopen("/proc/mounts", "r")) == NULL) {
-		logger(0, errno,  "unable to open /proc/mounts");
+		logger(-1, errno,  "unable to open /proc/mounts");
 		return -1;
 	}
+	path = realpath(root, NULL);
+	if (path == NULL)
+		path = strdup(root);
 	while (!feof(fp)) {
 		if (fgets(buf, sizeof(buf), fp) == NULL)
 			break;
 		if (sscanf(buf, "%*[^ ] %s ", mnt) != 1)
 			continue;
-		if (!strcmp(mnt, root)) {
+		if (!strcmp(mnt, path)) {
 			ret = 1;	
 			break;
 		}
 	}
+	free(path);
 	fclose(fp);
 	return ret;
 }
@@ -70,7 +82,7 @@ int vz_mount(fs_param *fs, int remount)
         if (mount(fs->private, fs->root, "simfs", mntopt,
 			remount ? "" : fs->private) < 0)
 	{
-                logger(0, errno, "Can't mount: %s %s", fs->root, fs->private);
+                logger(-1, errno, "Can't mount: %s %s", fs->root, fs->private);
                 return VZ_FS_CANTMOUNT;
         }
 	return 0;
