@@ -36,6 +36,7 @@
 #include "env.h"
 #include "script.h"
 #include "util.h"
+#include "vps_configure.h"
 
 
 int find_ip(list_head_t *ip_h, char *ipaddr)
@@ -94,87 +95,12 @@ int ip_ctl(vps_handler *h, envid_t veid, int op, char *ip)
 				ret = VZ_CANT_ADDIP;
 				break;
 		}
-		logger(0, errno, "Unable to %s IP %s",
+		logger(-1, errno, "Unable to %s IP %s",
 			op == VE_IP_ADD ? "add" : "del", ip);
 	}
 	return ret;
 }
 
-static struct vps_state{
-	char *name;
-	int id;
-} vps_states[] = {
-	{"starting", STATE_STARTING},
-	{"running", STATE_RUNNING},
-	{"running", STATE_RUNNING},
-	{"restoring", STATE_RESTORING},
-};
-
-const char *state2str(int state)
-{
-	int i;
-
-	for (i = 0; i < sizeof(vps_states) / sizeof(*vps_states); i++)
-		if (vps_states[i].id == state)
-			return vps_states[i].name;
-
-	return NULL;		
-}
-
-/*
- * Setup (add/delete) ip addreses inside VPS
- */
-int vps_ip_configure(vps_handler *h, envid_t veid, dist_actions *actions,
-	char *root, int op, net_param *net, int state)
-{
-	char *envp[5];
-	char *str;
-	const char *script = NULL;
-	int ret, i;
-	char vps_state[32];
-	const char *str_state;
-	const char *delall = "IPDELALL=yes";
-
-	if (list_empty(&net->ip) && !net->delall && state != STATE_STARTING) 
-		return 0;
-	if (actions == NULL)
-		return 0;
-	switch (op) {
-		case ADD:
-			script = actions->add_ip;
-			if (script == NULL) {
-				logger(0, 0, "Warning: add_ip action script"
-					" is not specified");
-				return 0;
-			}
-			break;
-		case DEL:
-			script = actions->del_ip;
-			if (script == NULL) {
-				logger(0, 0, "Warning: del_ip action script"
-					" is not specified");
-				return 0;
-			}
-			break;
-	}
-	i = 0;
-	str_state = state2str(state);
-	snprintf(vps_state, sizeof(vps_state), "VE_STATE=%s", str_state);
-	envp[i++] = vps_state;
-	str = list2str("IP_ADDR", &net->ip);
-	if (str != NULL)
-		envp[i++] = str;
-	if (net->delall)
-		envp[i++] = (char *) delall;
-	envp[i++] = ENV_PATH;
-	envp[i] = NULL;
-	ret = vps_exec_script(h, veid, root, NULL, envp, script, DIST_FUNC,
-		SCRIPT_EXEC_TIMEOUT);
-	if (str != NULL) free(str);
-
-	return ret;
-}
-
 int run_net_script(envid_t veid, int op, list_head_t *ip_h, int state,
 	int skip_arpdetect)
 {
@@ -323,14 +249,14 @@ int vps_netdev_ctl(vps_handler *h, envid_t veid, int op, net_param *net)
 	if (list_empty(dev_h))
 		return 0;
 	if (!vps_is_run(h, veid)){
-		logger(0, 0, "Unable to setup network devices:"
-			" VPS is not running");
+		logger(-1, 0, "Unable to setup network devices: "
+			"VE is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	cmd = (op == ADD) ? VE_NETDEV_ADD : VE_NETDEV_DEL;
 	list_for_each(dev, dev_h, list) {
 		if ((ret = netdev_ctl(h, veid, cmd, dev->val))) {
-			logger(0, errno, "Unable to %s netdev %s",
+			logger(-1, errno, "Unable to %s netdev %s",
 				(op == ADD) ? "add": "del", dev->val);
 			break;
 		}
@@ -345,14 +271,14 @@ int vps_net_ctl(vps_handler *h, envid_t veid, int op, net_param *net,
 	int ret = 0;
 
 	if (list_empty(ip_h) &&	!net->delall) {
-		/* make initial network setup on VPS start*/
+		/* make initial network setup on VE start*/
 		if (state == STATE_STARTING && op == ADD)
 			goto configure;
 		return 0;
 	}
 	if (!vps_is_run(h, veid)) {
-		logger(0, 0, "Unable to apply network parameters: VPS is not"
-			" running");
+		logger(-1, 0, "Unable to apply network parameters: "
+			"VE is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
 	if (op == ADD) {
@@ -378,7 +304,7 @@ static inline int get_vps_ip_proc(envid_t veid, list_head_t *ip_h)
 	int id, cnt = 0;
 
 	if ((fd = fopen(PROC_VEINFO, "r")) == NULL) {
-		logger(0, errno, "Unable to open %s", PROC_VEINFO);
+		logger(-1, errno, "Unable to open %s", PROC_VEINFO);
 		return -1;
 	}
 	while (!feof(fd)) {
@@ -423,7 +349,7 @@ static inline int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
 {
 	int ret = -1;
 	struct vzlist_veipv4ctl veip;
-	uint32_t *addr;
+	uint32_t *addr, *tmp;
 	char buf[16];
 	int i;
 
@@ -440,9 +366,12 @@ static inline int get_vps_ip_ioctl(vps_handler *h, envid_t veid,
 		else if (ret <= veip.num)
 			break;
 		veip.num = ret;
-		addr = realloc(addr, veip.num * sizeof(*veip.ip));
-		if (addr == NULL)
-			return -1;
+		tmp = realloc(addr, veip.num * sizeof(*veip.ip));
+		if (tmp == NULL) {
+			ret = -1;
+			goto out;
+		}
+		addr = tmp;
 	}
 	if (ret > 0) {
 		for (i = ret - 1; i >= 0; i--) {
