@@ -37,17 +37,18 @@
 #include "script.h"
 #include "fs.h"
 
+volatile sig_atomic_t alarm_flag;
 static char *envp_bash[] = {"HOME=/", "TERM=linux", "PATH=/bin:/sbin:/usr/bin:/usr/sbin:", NULL};
 
 int read_script(const char *fname, char *include, char **buf)
 {
 	struct stat st;
-	char *p = NULL;
+	char *tmp, *p = NULL;
 	int  fd, len = 0;
 	char *inc;
 
 	if (!fname) {
-		logger(0, 0, "read_script: file name not specified");
+		logger(-1, 0, "read_script: file name not specified");
 		return -1;
 	}
 	/* Read include file first */
@@ -66,17 +67,18 @@ int read_script(const char *fname, char *include, char **buf)
 			return -1;
 	}
 	if (stat(fname, &st)) {
-		logger(0, 0, "file %s not found", fname);
+		logger(-1, 0, "file %s not found", fname);
 		return -1;
 	}
 	if ((fd = open(fname, O_RDONLY)) < 0) {
-		logger(0, errno, "Unable to open %s", fname);
+		logger(-1, errno, "Unable to open %s", fname);
 		goto err;
 	}
 	if (*buf != NULL) {
-		*buf = realloc(*buf, st.st_size + len + 2);
-		if (*buf ==  NULL)
+		tmp = realloc(*buf, st.st_size + len + 2);
+		if (tmp ==  NULL)
 			goto err;
+		*buf = tmp;
 		p = *buf + len;
 	} else {
 		*buf = malloc(st.st_size + 2);
@@ -85,7 +87,7 @@ int read_script(const char *fname, char *include, char **buf)
 		p = *buf;
 	}
 	if ((len =  read(fd, p, st.st_size)) < 0) {
-		logger(0, errno, "Error reading %s", fname);
+		logger(-1, errno, "Error reading %s", fname);
 		goto err;
 	}
 	p += len;
@@ -102,17 +104,19 @@ err:
 	return -1;
 }
 
-int run_script(const char *f, char *argv[], char *envp[], int quiet)
+#define ENV_SIZE	256
+int run_script(const char *f, char *argv[], char *env[], int quiet)
 {
 	int child, pid, fd;
 	int status;
-	int ret;
+	int ret, i;
 	char *cmd;
 	struct sigaction act, actold;
 	int out[2];
+	char *envp[ENV_SIZE];
 
 	if (!stat_file(f)) {
-		logger(0, 0, "File %s not found", f);
+		logger(-1, 0, "File %s not found", f);
 		return VZ_NOSCRIPT;
 	}
 	sigaction(SIGCHLD, NULL, &actold);
@@ -127,9 +131,17 @@ int run_script(const char *f, char *argv[], char *envp[], int quiet)
 		free(cmd);
 	}
 	if (quiet && pipe(out) < 0) {
-		logger(0, errno, "run_script: unable to create pipe");
+		logger(-1, errno, "run_script: unable to create pipe");
 		return -1;
 	}
+	i = 0;
+	if (env != NULL) {
+		for (i = 0; i < ENV_SIZE - 1 && env[i] != NULL; i++)
+			envp[i] = env[i];
+	}
+	for (; i < ENV_SIZE - 1 && envp_bash[i] != NULL; i++)
+		envp[i] = envp_bash[i];
+	envp[i] = NULL;
 	if ((child = fork()) == 0) {
 		fd = open("/dev/null", O_WRONLY);
 		if (fd < 0)
@@ -148,11 +160,11 @@ int run_script(const char *f, char *argv[], char *envp[], int quiet)
 			close(out[1]);
 */
 		}
-		execve(f, argv, envp != NULL ? envp : envp_bash);
-		logger(0, errno, "Error exec %s", f);
+		execve(f, argv, envp);
+		logger(-1, errno, "Error exec %s", f);
 		exit(1);
 	} else if(child == -1) {
-		logger(0, errno, "Unable to fork");
+		logger(-1, errno, "Unable to fork");
 		ret = VZ_RESOURCE_ERROR;
 		goto err;
 	}
@@ -164,10 +176,10 @@ int run_script(const char *f, char *argv[], char *envp[], int quiet)
 		if (WIFEXITED(status))
 			ret = WEXITSTATUS(status);
 		else if (WIFSIGNALED(status))
-			logger(0, 0, "Received signal:"
+			logger(-1, 0, "Received signal:"
 					"  %d in %s", WTERMSIG(status), f);
 	} else {
-		logger(0, errno, "Error in waitpid");
+		logger(-1, errno, "Error in waitpid");
 	}
 err:
 	sigaction(SIGCHLD, &actold, NULL);
@@ -228,7 +240,7 @@ int mk_reboot_script()
 		return 1;
 	sprintf(buf, "%s/" VZREBOOT, rc);
 	if ((fd = open(buf, O_CREAT|O_WRONLY|O_TRUNC, 0755)) < 0) {
-		logger(0, errno, "Unable to create %s", buf);
+		logger(-1, errno, "Unable to create %s", buf);
 		return 1;
 	}
 	write(fd, REBOOT_SCRIPT, sizeof(REBOOT_SCRIPT));
@@ -247,7 +259,7 @@ int mk_quota_link()
 	char buf[64];
 
 	if (stat("/", &st)) {
-		logger(0, errno, "Unable to stat /");
+		logger(-1, errno, "Unable to stat /");
 		return -1;
 	}
 	fs = vz_fs_get_name();
@@ -256,16 +268,106 @@ int mk_quota_link()
 	unlink(buf);
 	logger(3, 0, "Setup quota dev %s", buf);
 	if (mknod(buf, S_IFBLK | S_IXGRP, st.st_dev))
-		logger(0, errno, "Unable to create %s", buf);
+		logger(-1, errno, "Unable to create %s", buf);
 	snprintf(buf, sizeof(buf), PROC_QUOTA "%08lx" QUOTA_U,
 		(unsigned long)st.st_dev);
 	unlink(QUOTA_U);
 	if (symlink(buf, QUOTA_U)) 
-		logger(0, errno, "Unable to create symlink %s", buf); 
+		logger(-1, errno, "Unable to create symlink %s", buf); 
 	snprintf(buf, sizeof(buf), PROC_QUOTA "%08lx" QUOTA_G,
 		(unsigned long)st.st_dev);
 	unlink(QUOTA_G);	
 	if (symlink(buf, QUOTA_G))
-		logger(0, errno, "Unable to create symlink %s", buf); 
+		logger(-1, errno, "Unable to create symlink %s", buf); 
 	return 0;
 }
+
+#define INITTAB_FILE		"/etc/inittab"
+#define INITTAB_VZID		"vz:"
+#define INITTAB_ACTION		INITTAB_VZID "2345:once:sh -c 'if [ -p "VZFIFO_FILE" ]; then echo done > " VZFIFO_FILE "; fi'\n"
+#define MAX_WAIT_TIMEOUT	 60 * 60
+
+int add_reach_runlevel_mark()
+{
+	int fd, found, len, ret;
+	char buf[4096];
+
+	unlink(VZFIFO_FILE);
+	if (mkfifo(VZFIFO_FILE, 0644)) {
+		fprintf(stderr, "Unable to create " VZFIFO_FILE " %s\n",
+			strerror(errno));
+		return -1;
+	}
+	if ((fd = open(INITTAB_FILE, O_RDWR | O_APPEND)) == -1) {
+		fprintf(stderr, "Unable to open " INITTAB_FILE " %s\n",
+			strerror(errno));
+		return -1;
+	}
+	ret = 0;
+	found = 0;
+	while (1) { 
+		len = read(fd, buf, sizeof(buf));
+		if (len == 0)
+			break;
+		if (len < 0) {
+			fprintf(stderr, "Unable to read from " INITTAB_FILE
+				" %s\n",
+				strerror(errno));
+			ret = -1;
+			break;
+		}
+		buf[len] = 0;
+		if (strstr(buf, "\n" INITTAB_VZID) != NULL) {
+			found = 1;
+			break;
+		}
+	}
+	if (!found) {
+		if (write(fd, INITTAB_ACTION, sizeof(INITTAB_ACTION) - 1) == -1)
+		{
+			fprintf(stderr, "Unable to write to " INITTAB_FILE
+				" %s\n",
+				strerror(errno));
+			ret = -1;
+		}
+	}
+	close(fd);
+	return ret;
+}
+
+static void alarm_handler(int sig)
+{
+	alarm_flag = 1;
+}
+
+int wait_on_fifo(void *data)
+{
+	int fd, buf, ret;
+	struct sigaction act, actold;
+
+	ret = 0;
+	alarm_flag = 0;
+	act.sa_flags = 0;
+	act.sa_handler = alarm_handler;
+	sigemptyset(&act.sa_mask);
+	sigaction(SIGALRM, &act, &actold);
+
+	alarm(MAX_WAIT_TIMEOUT);
+	fd = open(VZFIFO_FILE, O_RDONLY);
+	if (fd == -1) {
+		fprintf(stderr, "Unable to open " VZFIFO_FILE " %s\n",
+			strerror(errno));
+		ret = -1;
+		goto err;
+	}
+	if (read(fd, &buf, sizeof(buf)) == -1)
+		ret = -1;
+err:
+	if (alarm_flag)
+		ret = VZ_EXEC_TIMEOUT;
+	alarm(0);
+	sigaction(SIGALRM, &actold, NULL);
+	unlink(VZFIFO_FILE);
+	return ret;
+}
+
