@@ -32,6 +32,7 @@
 
 #include "util.h"
 #include "logger.h"
+#include "fs.h"
 
 #ifndef NR_OPEN
 #define NR_OPEN 1024
@@ -122,7 +123,7 @@ int make_dir(char *path, int full)
 		ps = p + 1;
 		if (!stat_file(buf)) {
 			if (mkdir(buf, 0755)) {
-				logger(0, errno, "Can't create directory %s",
+				logger(-1, errno, "Can't create directory %s",
 					buf);
 				return 1;
 			}
@@ -132,7 +133,7 @@ int make_dir(char *path, int full)
 		return 0;
 	if (!stat_file(path)) {
 		if (mkdir(path, 0755)) {
-			logger(0, errno, "Can't create directory %s", path);
+			logger(-1, errno, "Can't create directory %s", path);
 			return 1;
 		}
 	}
@@ -179,7 +180,7 @@ int check_var(const void *val, const char *message)
 {
         if (val != NULL)
                 return 0;
-        logger(0, 0, "%s", message);
+        logger(-1, 0, "%s", message);
 
         return 1;
 }
@@ -191,16 +192,16 @@ int cp_file(char *dst, char *src)
 	char buf[4096];
 
 	if (stat(src, &st) < 0) {
-		logger(0, errno, "Unable to stat %s", src);
+		logger(-1, errno, "Unable to stat %s", src);
 		return -1;
 	}
 	len = st.st_size;
 	if ((fd_src = open(src, O_RDONLY)) < 0) {
-		logger(0, errno, "Unable to open %s", src);
+		logger(-1, errno, "Unable to open %s", src);
 		return -1;
 	}
 	if ((fd_dst = open(dst, O_CREAT|O_RDWR, st.st_mode)) < 0) {
-		logger(0, errno, "Unable to open %s", dst);
+		logger(-1, errno, "Unable to open %s", dst);
 		close(fd_src);
 		return -1;
 	}
@@ -209,12 +210,12 @@ int cp_file(char *dst, char *src)
 		if (!ret)
 			break;
 		else if (ret < 0) {
-			logger(0, errno, "Unable to read from %s", src);
+			logger(-1, errno, "Unable to read from %s", src);
 			ret = -1;
 			break;
 		}
 		if (write(fd_dst, buf, len) < 0) {
-			logger(0, errno, "Unable write to %s", dst);
+			logger(-1, errno, "Unable write to %s", dst);
 			ret = -1;
 			break;
 		}
@@ -356,7 +357,7 @@ int get_mem(unsigned long long *mem)
 {
 	long pagesize;
 	if ((*mem = sysconf(_SC_PHYS_PAGES)) == -1) {
-		logger(0, errno, "Unable to get total phys pages");
+		logger(-1, errno, "Unable to get total phys pages");
 		return -1;
 	}
 	if ((pagesize = get_pagesize()) < 0)
@@ -373,7 +374,7 @@ int get_thrmax(int *thrmax)
 	if (thrmax == NULL)
 		return 1;
 	if ((fd = fopen(PROCTHR, "r")) == NULL) {
-		logger(0, errno, "Unable to open " PROCTHR);
+		logger(-1, errno, "Unable to open " PROCTHR);
 		return 1;
 	}
 	if (fgets(str, sizeof(str), fd) == NULL) {
@@ -392,7 +393,7 @@ int get_swap(unsigned long long *swap)
 	char str[128];
 
 	if ((fd = fopen(PROCMEM, "r")) == NULL)	{
-		logger(0, errno, "Cannot open " PROCMEM);
+		logger(-1, errno, "Cannot open " PROCMEM);
 		return -1;
 	}
 	while (fgets(str, sizeof(str), fd)) {
@@ -402,19 +403,37 @@ int get_swap(unsigned long long *swap)
 			return 0;
 		}
 	}
-	logger(0, errno, "Swap: is not found in " PROCMEM );
+	logger(-1, errno, "Swap: is not found in " PROCMEM );
 	fclose(fd);
 
 	return -1;
 }
 
+int get_num_cpu()
+{
+	FILE *fd;
+	char str[128];
+	int ncpu = 0;
+
+	if ((fd = fopen("/proc/cpuinfo", "r")) == NULL)	{
+		logger(-1, errno, "Cannot open /proc/cpuinfo");
+		return 1;
+	}
+	while (fgets(str, sizeof(str), fd)) {
+		if (!strncmp(str, "processor", 9))
+			ncpu++;
+	}
+	fclose(fd);
+	return !ncpu ? 1 : ncpu;
+}
+
 int get_lowmem(unsigned long long *mem)
 {
 	FILE *fd;
 	char str[128];
 
 	if ((fd = fopen(PROCMEM, "r")) == NULL)	{
-		logger(0, errno, "Cannot open " PROCMEM);
+		logger(-1, errno, "Cannot open " PROCMEM);
 		return -1;
 	}
 	while (fgets(str, sizeof(str), fd)) {
@@ -424,7 +443,7 @@ int get_lowmem(unsigned long long *mem)
 			return 0;
 		}
 	}
-	logger(0, errno, "LowTotal: is not found in" PROCMEM);
+	logger(-1, errno, "LowTotal: is not found in" PROCMEM);
 	fclose(fd);
 	return -1;
 }
@@ -524,3 +543,35 @@ void close_fds(int close_std, ...)
 		close(fd);
 	}
 }
+
+/* Renames (to "*.destroyed" if action == MOVE) or removes config,
+ * (if action == DESTR)
+ * Also, appropriate mount/umount scripts are linked.
+ */
+int move_config(int veid, int action)
+{
+	char conf[PATH_LEN];
+	char newconf[PATH_LEN];
+
+	snprintf(conf, sizeof(conf), VPS_CONF_DIR "%d.conf", veid);
+	snprintf(newconf, sizeof(newconf), "%s." DESTR_PREFIX, conf);
+	action == BACKUP ? rename(conf, newconf) : unlink(newconf);
+
+	snprintf(conf, sizeof(conf), VPS_CONF_DIR "%d." MOUNT_PREFIX, veid);
+	snprintf(newconf, sizeof(newconf), "%s." DESTR_PREFIX, conf);
+	action == BACKUP ? rename(conf, newconf) : unlink(newconf);
+	
+	snprintf(conf, sizeof(conf), VPS_CONF_DIR "%d." UMOUNT_PREFIX, veid);
+	snprintf(newconf, sizeof(newconf), "%s." DESTR_PREFIX, conf);
+	action == BACKUP ? rename(conf, newconf) : unlink(newconf);
+
+	snprintf(conf, sizeof(conf), VPS_CONF_DIR "%d." START_PREFIX, veid);
+	snprintf(newconf, sizeof(newconf), "%s." DESTR_PREFIX, conf);
+	action == BACKUP ? rename(conf, newconf) : unlink(newconf);
+
+	snprintf(conf, sizeof(conf), VPS_CONF_DIR "%d." STOP_PREFIX, veid);
+	snprintf(newconf, sizeof(newconf), "%s." DESTR_PREFIX, conf);
+	action == BACKUP ? rename(conf, newconf) : unlink(newconf);
+
+	return 0;
+}
