@@ -55,17 +55,17 @@ void usage()
 	struct mod_action mod;
 
 	version();
-	fprintf(stdout, "Copyright (C) 2000-2005 SWsoft.\n");
+	fprintf(stdout, "Copyright (C) 2000-2006 SWsoft.\n");
 	fprintf(stdout, "This program may be distributed under the terms of the GNU GPL License.\n\n");
 	fprintf(stdout, "Usage: vzctl [options] <command> <veid> [parameters]\n"
 "vzctl destroy | mount | umount | stop | status | enter <veid>\n"
 "vzctl create <veid> {--ostemplate <name>] [--config <name>]\n"
 "   [--private <path>] [--root <path>] [--ipadd <addr>] | [--hostname <name>]\n"
-"vzctl start <veid> [--force]\n"
+"vzctl start <veid> [--force] [--wait]\n"
 "vzctl exec | exec2 <veid> <command> [arg ...]\n"
 "vzctl runscript <veid> <script>\n"
-"vzctl chkpnt <vpsid> [--dumpfile <name>]\n"
-"vzctl restore <vpsid> [--dumpfile <name>]\n"
+"vzctl chkpnt <veid> [--dumpfile <name>]\n"
+"vzctl restore <veid> [--dumpfile <name>]\n"
 "vzctl set veid [--save] [--setmode restart|ignore]\n"
 "   [--ipadd <addr>] [--ipdel <addr>|all] [--hostname <name>]\n"
 "   [--nameserver <addr>] [--searchdomain <name>] [--onboot yes|no]\n"
@@ -75,9 +75,10 @@ void usage()
 "   [--noatime yes|no] [--capability <name>:on|off ...]\n"
 "   [--devices b|c:major:minor|all:r|w|rw]\n"
 "   [--devnodes device:r|w|rw|none]\n"
-"   [--veth_add dev_name,dev_addr,dev_name_vps,dev_addr_vps]\n"
+"   [--veth_add dev_name,dev_addr,dev_name_ve,dev_addr_ve]\n"
 "   [--veth_del dev_name]\n"
-"   [--applyconfig <name>]\n");
+"   [--applyconfig <name>]\n"
+"   [--name <vename>]\n");
  
 	fprintf(stdout, "   [--iptables <name>] [--disabled <yes|no>]\n");
 	fprintf(stdout, "   [UBC parameters]\n"
@@ -108,6 +109,7 @@ int main(int argc, char *argv[], char *envp[])
 	vps_param *gparam = NULL, *vps_p = NULL, *cmd_p = NULL;
 	const char *action_nm;
 	struct sigaction act;
+	char *name = NULL;
 
 	_proc_title = argv[0];
 	_proc_title_len = envp[0] - argv[0];
@@ -205,14 +207,15 @@ int main(int argc, char *argv[], char *envp[])
 		}
 	}
 	if (argc < 3) {
-		fprintf(stderr, "VPS id is not given\n");
+		fprintf(stderr, "VE id is not given\n");
 		ret = VZ_INVALID_PARAMETER_VALUE;
 		goto error;
 	}
 	if (parse_int(argv[2], &veid)) {
-		veid = get_veid_by_name(argv[2]);
+		name = strdup(argv[2]);
+		veid = get_veid_by_name(name);
 		if (veid < 0) {
-			fprintf(stderr, "Bad VPS id %s\n", argv[2]);
+			fprintf(stderr, "Bad VE id %s\n", argv[2]);
 			ret = VZ_INVALID_PARAMETER_VALUE;
 			goto error;
 		}
@@ -238,22 +241,31 @@ int main(int argc, char *argv[], char *envp[])
 		ret = VZ_INVALID_PARAMETER_VALUE;
 		goto error;
 	} else if (veid < 0) {
-		fprintf(stderr, "Bad VPS id %d\n", veid);
+		fprintf(stderr, "Bad VE id %d\n", veid);
 		ret = VZ_INVALID_PARAMETER_VALUE;
 		goto error;
 	}
 	get_vps_conf_path(veid, buf, sizeof(buf));
 	if (stat_file(buf)) {
 		if (vps_parse_config(veid, buf, vps_p, &g_action)) {
-			logger(0, 0, "Error in config file %s",
+			logger(-1, 0, "Error in config file %s",
 				buf);
-	                exit(VZ_NOCONFIG);
+			ret = VZ_NOCONFIG;
+			goto error;
         	}
+		if (name != NULL &&
+		    vps_p->res.name.name != NULL &&
+		    strcmp(name, vps_p->res.name.name))
+		{
+			logger(-1, 0, "Unable to find VE by name %s", name);
+			ret = VZ_INVALID_PARAMETER_VALUE;
+			goto error;
+		}
 	} else if (action != ACTION_CREATE &&
 			action != ACTION_STATUS &&
 			action != ACTION_SET)
 	{
-		logger(0, 0, "VPS config file does not exist");
+		logger(-1, 0, "VE config file does not exist");
 		ret = VZ_NOVECONFIG;
 		goto error;
 	}
@@ -268,6 +280,7 @@ error:
 	free_vps_param(vps_p);
 	free_vps_param(cmd_p);
 	free_log();
+	if (name != NULL) free(name);
 
 	return ret;
 }
