@@ -48,13 +48,14 @@ static int n_veinfo = 0;
 
 static char g_outbuffer[4096] = "";
 static char *p_outbuffer = g_outbuffer;
+static char *e_buf = g_outbuffer + sizeof(g_outbuffer) - 1;
 static char *host_pattern = NULL;
 static char *name_pattern = NULL;
 static int vzctlfd;
 static struct Cfield_order *g_field_order = NULL;
 struct Cfield_order *last_field = NULL;
-static char *default_field_order = "vpsid,numproc,status,ip,hostname";
-static char *default_nm_field_order = "vpsid,numproc,status,ip,name";
+static char *default_field_order = "veid,numproc,status,ip,hostname";
+static char *default_nm_field_order = "veid,numproc,status,ip,name";
 static int g_sort_field = 0;
 static int *g_ve_list = NULL;
 static int n_ve_list = 0;
@@ -90,29 +91,29 @@ static void print_ip(struct Cveinfo *p, int index);
 /* Print functions */
 static void print_veid(struct Cveinfo *p, int index)
 {
-	p_outbuffer += sprintf(p_outbuffer, "%10d", p->veid);
+	p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10d", p->veid);
 }
 
 static void print_status(struct Cveinfo *p, int index)
 {
-	p_outbuffer += sprintf(p_outbuffer, "%-7s", ve_status[p->status]);
+	p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%-7s", ve_status[p->status]);
 }
 
 static void print_laverage(struct Cveinfo *p, int index)
 {
 	if (p->la == NULL)
-		p_outbuffer += sprintf(p_outbuffer, "%14s", "-");
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%14s", "-");
 	else
-		p_outbuffer += sprintf(p_outbuffer, "%1.2f/%1.2f/%1.2f",
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%1.2f/%1.2f/%1.2f",
 			p->la->la[0], p->la->la[1], p->la->la[2]);
 }
 
 static void print_cpulimit(struct Cveinfo *p, int index)
 {
 	if (p->cpu == NULL)
-		p_outbuffer += sprintf(p_outbuffer, "%7s", "-");
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%7s", "-");
 	else
-		p_outbuffer += sprintf(p_outbuffer, "%7lu",
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%7lu",
 			p->cpu->limit[index]);
 }
 
@@ -122,9 +123,9 @@ static void fn(struct Cveinfo *p, int index)		  		\
 	if (p->res == NULL ||						\
 		(p->status != VE_RUNNING &&				\
 			(index == 0 || index == 1 || index == 4)))	\
-		p_outbuffer += sprintf(p_outbuffer, "%10s", "-");	\
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10s", "-");	\
 	else								\
-		p_outbuffer += sprintf(p_outbuffer, "%10lu",		\
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10lu",		\
 					p->res->name[index]);		\
 }									\
 
@@ -154,9 +155,9 @@ static void fn(struct Cveinfo *p, int index)		  		\
 {									\
 	if (p->res == NULL ||						\
 		(p->status != VE_RUNNING && (index == 0)))		\
-		p_outbuffer += sprintf(p_outbuffer, "%10s", "-");	\
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10s", "-");	\
 	else								\
-		p_outbuffer += sprintf(p_outbuffer, "%10lu",		\
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10lu",		\
 					p->res->name[index]);		\
 }									\
 
@@ -373,7 +374,9 @@ SORT_UL_RES(cpuunits_sort_fn, Ccpu, cpu, limit, 1)
 struct Cfield field_names[] = 
 {
 /* veid should have index 0 */
-{"vpsid" , "VPSID", "%10s",  0, RES_NONE, print_veid, id_sort_fn},
+{"veid", "VEID", "%10s", 0,  RES_NONE, print_veid, id_sort_fn},
+/* vpsid is for backward compatibility -- will be removed later */
+{"vpsid" , "VEID", "%10s",  0, RES_NONE, print_veid, id_sort_fn},
 {"hostname", "HOSTNAME", "%-32s", 0, RES_HOSTNAME, print_hostname, hostnm_sort_fn},
 {"name", "NAME", "%-32s", 0, RES_NAME, print_name, name_sort_fn},
 {"ip", "IP_ADDR", "%-15s", 0, RES_IP, print_ip, ip_sort_fn},
@@ -519,7 +522,7 @@ static void print_hostname(struct Cveinfo *p, int index)
 
 	if (p->hostname != NULL)
 		str = p->hostname;
-	r = sprintf(p_outbuffer, "%-32s", str);
+	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-32s", str);
 	if (last_field != NULL && 
 		field_names[last_field->order].res_type != RES_HOSTNAME)
 	{
@@ -535,7 +538,7 @@ static void print_name(struct Cveinfo *p, int index)
 
 	if (p->name != NULL)
 		str = p->name;
-	r = sprintf(p_outbuffer, "%-32s", str);
+	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-32s", str);
 	if (last_field != NULL &&
 		field_names[last_field->order].res_type != RES_NAME)
 	{
@@ -559,7 +562,7 @@ static void print_ip(struct Cveinfo *p, int index)
 		if ((ch = strchr(str, ' ')) != NULL)
 			*ch = 0;
 	}
-	r = sprintf(p_outbuffer, "%-15s", str);
+	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-15s", str);
 	if (last_field != NULL && 
 		field_names[last_field->order].res_type != RES_IP)
 	{
@@ -580,19 +583,21 @@ void *x_malloc(int size)
 
 void *x_realloc(void *ptr, int size)
 {
-	if ((ptr = realloc(ptr, size)) == NULL) {
+	void *tmp;
+
+	if ((tmp = realloc(ptr, size)) == NULL) {
 		printf("Error: unable to allocate %d bytes\n", size);
 		exit(1);
 	}
-	return ptr;
+	return tmp;
 }
 
 void usage()
 {
 	fprintf(stderr, "Usage: vzlist [-a] [-o name[,name...]] [-s {name|-name}] [-h <pattern>] [-N <pattern>]\n");
-	fprintf(stderr, "\t\t[-H] [-S] [vpsid [vpsid ...]|-1]\n");
+	fprintf(stderr, "\t\t[-H] [-S] [veid [veid ...]|-1]\n");
 	fprintf(stderr, "       vzlist -L\n\n");
-	fprintf(stderr, "\t--all -a\t list of all VPS\n");
+	fprintf(stderr, "\t--all -a\t list of all VEs\n");
 	fprintf(stderr, "\t--output -o\t output only specified parameters\n");
 	fprintf(stderr, "\t--hostname -h\t hostname search pattern\n");
 	fprintf(stderr, "\t--name -n\t display VE name\n");
@@ -607,7 +612,7 @@ void usage()
 
 int id_search_fn(const void* val1, const void* val2)
 {
-	return ((int)val1 - ((struct Cveinfo*)val2)->veid);
+	return (*(int *)val1 - ((struct Cveinfo*)val2)->veid);
 }
 
 int veid_search_fn(const void* val1, const void* val2)
@@ -689,6 +694,8 @@ void print_ve()
 			f = p->order;
 			field_names[f].print_fn(&veinfo[idx],
 						field_names[f].index);
+			if (p_outbuffer >= e_buf)
+				break;
 			if (p->next != NULL)
 				*p_outbuffer++ = ' ';
 		}
@@ -708,7 +715,7 @@ void add_elem(struct Cveinfo *ve)
 
 inline struct Cveinfo *find_ve(int veid)
 {
-	return (struct Cveinfo *) bsearch((void*)veid, veinfo, n_veinfo,
+	return (struct Cveinfo *) bsearch(&veid, veinfo, n_veinfo,
 			sizeof(struct Cveinfo), id_search_fn);
 }
 
@@ -1482,7 +1489,7 @@ int collect()
 		return ret;
 	/* No VE found, exit with error */
 	if (!n_veinfo) {
-		fprintf(stderr, "VPS not found\n");
+		fprintf(stderr, "VE not found\n");
 		return 1;
 	}
 	if (check_param(RES_QUOTA)) 
@@ -1532,7 +1539,8 @@ void free_veinfo()
 			free(veinfo[i].cpu);
 		if (veinfo[i].ve_root != NULL)
 			free(veinfo[i].ve_root);
-
+		if (veinfo[i].ve_private != NULL)
+			free(veinfo[i].ve_private);
 	}
 }
 
@@ -1636,5 +1644,9 @@ int main(int argc, char **argv)
 		return ret;
 	print_ve();
 	free_veinfo();
+	if (host_pattern != NULL) free(host_pattern);
+	if (name_pattern != NULL) free(name_pattern);
+	if (f_order != NULL) free(f_order);
+
 	return 0;
 }
